@@ -32,7 +32,7 @@ static void usePotentialyUnusedSymbols()
     // called at runtime.
     volatile bool doIt = false;
     if (doIt)
-        emscripten_set_wheel_callback(NULL, 0, 0, NULL);
+        emscripten_set_wheel_callback("", 0, 0, NULL);
 }
 
 Q_CONSTRUCTOR_FUNCTION(usePotentialyUnusedSymbols)
@@ -365,10 +365,13 @@ void WebPromiseManager::adoptPromise(emscripten::val target, PromiseCallbacks ca
 #if defined(QT_STATIC)
 
 EM_JS(bool, jsHaveAsyncify, (), { return typeof Asyncify !== "undefined"; });
+EM_JS(bool, jsHaveJspi, (),
+      { return typeof Asyncify !== "undefined" && !!Asyncify.makeAsyncFunction && !!WebAssembly.Function; });
 
 #else
 
 bool jsHaveAsyncify() { return false; }
+bool jsHaveJspi() { return false; }
 
 #endif
 
@@ -498,7 +501,7 @@ uint32_t ArrayBuffer::byteLength() const
     return m_arrayBuffer["byteLength"].as<uint32_t>();
 }
 
-emscripten::val ArrayBuffer::val()
+emscripten::val ArrayBuffer::val() const
 {
     return m_arrayBuffer;
 }
@@ -531,7 +534,7 @@ Blob Blob::copyFrom(const char *buffer, uint32_t size)
     return copyFrom(buffer, size, "application/octet-stream");
 }
 
-emscripten::val Blob::val()
+emscripten::val Blob::val() const
 {
     return m_blob;
 }
@@ -582,7 +585,7 @@ std::string File::type() const
     return m_file["type"].as<std::string>();
 }
 
-emscripten::val File::val()
+emscripten::val File::val() const
 {
     return m_file;
 }
@@ -641,7 +644,7 @@ void FileReader::onAbort(const std::function<void(emscripten::val)> &onAbort)
     m_onAbort = std::make_unique<EventCallback>(m_fileReader, "abort", onAbort);
 }
 
-emscripten::val FileReader::val()
+emscripten::val FileReader::val() const
 {
     return m_fileReader;
 }
@@ -739,7 +742,7 @@ Uint8Array Uint8Array::copyFrom(const QByteArray &buffer)
     return copyFrom(buffer.constData(), buffer.size());
 }
 
-emscripten::val Uint8Array::val()
+emscripten::val Uint8Array::val() const
 {
     return m_uint8Array;
 }
@@ -857,6 +860,12 @@ bool haveAsyncify()
     return HaveAsyncify;
 }
 
+bool haveJspi()
+{
+    static bool HaveJspi = jsHaveJspi();
+    return HaveJspi;
+}
+
 std::shared_ptr<CancellationFlag>
 readDataTransfer(emscripten::val webDataTransfer, std::function<QVariant(QByteArray)> imageReader,
                  std::function<void(std::unique_ptr<QMimeData>)> onDone)
