@@ -3,17 +3,18 @@
 
 #include "qplatformdefs.h"
 #include "private/qdatetimeparser_p.h"
-#include "private/qstringiterator_p.h"
 
 #include "qdatastream.h"
-#include "qset.h"
-#include "qvarlengtharray.h"
-#include "qlocale.h"
 #include "qdatetime.h"
-#if QT_CONFIG(timezone)
-#include "qtimezone.h"
-#endif
 #include "qdebug.h"
+#include "qlocale.h"
+#include "qset.h"
+#include "qtimezone.h"
+#include "qvarlengtharray.h"
+#include "private/qlocale_p.h"
+
+#include "private/qstringiterator_p.h"
+#include "private/qtenvironmentvariables_p.h"
 
 //#define QDATETIMEPARSER_DEBUG
 #if defined (QDATETIMEPARSER_DEBUG) && !defined(QT_NO_DEBUG_STREAM)
@@ -134,9 +135,7 @@ bool QDateTimeParser::setDigit(QDateTime &v, int index, int newVal) const
     int minute = time.minute();
     int second = time.second();
     int msec = time.msec();
-    Qt::TimeSpec tspec = v.timeSpec();
-    // Only offset from UTC is amenable to setting an int value:
-    int offset = tspec == Qt::OffsetFromUTC ? v.offsetFromUtc() : 0;
+    QTimeZone timeZone = v.timeRepresentation();
 
     const SectionNode &node = sectionNodes.at(index);
     switch (node.type) {
@@ -168,8 +167,8 @@ bool QDateTimeParser::setDigit(QDateTime &v, int index, int newVal) const
     case TimeZoneSection:
         if (newVal < absoluteMin(index) || newVal > absoluteMax(index))
             return false;
-        tspec = Qt::OffsetFromUTC;
-        offset = newVal;
+        // Only offset from UTC is amenable to setting an int value:
+        timeZone = QTimeZone::fromSecondsAheadOfUtc(newVal);
         break;
     case AmPmSection: hour = (newVal == 0 ? hour % 12 : (hour % 12) + 12); break;
     default:
@@ -209,12 +208,7 @@ bool QDateTimeParser::setDigit(QDateTime &v, int index, int newVal) const
     if (!newDate.isValid() || !newTime.isValid())
         return false;
 
-    // Preserve zone:
-    v =
-#if QT_CONFIG(timezone)
-         tspec == Qt::TimeZone ? QDateTime(newDate, newTime, v.timeZone()) :
-#endif
-         QDateTime(newDate, newTime, tspec, offset);
+    v = QDateTime(newDate, newTime, timeZone);
     return true;
 }
 
@@ -231,11 +225,7 @@ int QDateTimeParser::absoluteMax(int s, const QDateTime &cur) const
     const SectionNode &sn = sectionNode(s);
     switch (sn.type) {
     case TimeZoneSection:
-#if QT_CONFIG(timezone)
         return QTimeZone::MaxUtcOffsetSecs;
-#else
-        return +14 * 3600;  // NB: copied from QTimeZone
-#endif
     case Hour24Section:
     case Hour12Section:
         // This is special-cased in parseSection.
@@ -279,11 +269,7 @@ int QDateTimeParser::absoluteMin(int s) const
     const SectionNode &sn = sectionNode(s);
     switch (sn.type) {
     case TimeZoneSection:
-#if QT_CONFIG(timezone)
         return QTimeZone::MinUtcOffsetSecs;
-#else
-        return -14 * 3600;  // NB: copied from QTimeZone
-#endif
     case Hour24Section:
     case Hour12Section:
     case MinuteSection:
@@ -386,9 +372,9 @@ static qsizetype digitCount(QStringView str)
   not escaped and removes the escaping on those that are escaped
 
 */
-
 static QString unquote(QStringView str)
 {
+    // ### Align unquoting format strings for both from/toString(), QTBUG-110669
     const QLatin1Char quote('\'');
     const QLatin1Char slash('\\');
     const QLatin1Char zero('0');
@@ -413,14 +399,10 @@ static QString unquote(QStringView str)
 static inline int countRepeat(QStringView str, int index, int maxCount)
 {
     str = str.sliced(index);
-    if (maxCount > str.size())
-        maxCount = str.size();
+    if (maxCount < str.size())
+        str = str.first(maxCount);
 
-    const QChar ch(str[0]);
-    int count = 1;
-    while (count < maxCount && str[count] == ch)
-        ++count;
-    return count;
+    return qt_repeatCount(str);
 }
 
 static inline void appendSeparator(QStringList *list, QStringView string,
@@ -501,10 +483,11 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
 
             case 'z':
                 if (parserType != QMetaType::QDate) {
-                    const SectionNode sn = { MSecSection, i - add, countRepeat(newFormat, i, 3) < 3 ? 1 : 3, 0 };
+                    const int repeat = countRepeat(newFormat, i, 3);
+                    const SectionNode sn = { MSecSection, i - add, repeat < 3 ? 1 : 3, 0 };
                     newSectionNodes.append(sn);
                     appendSeparator(&newSeparators, newFormat, index, i - index, lastQuote);
-                    i += sn.count - 1;
+                    i += repeat - 1;
                     index = i + 1;
                     newDisplay |= MSecSection;
                 }
@@ -566,10 +549,8 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
                 break;
             case 't':
                 if (parserType == QMetaType::QDateTime) {
-                    // TODO (in qlocale.cpp's serialization, too) QTBUG-95966:
-                    // decide what different lengths of 't' format should do,
-                    // instead of repetition !
-                    const SectionNode sn = { TimeZoneSection, i - add, 1, 0 };
+                    const SectionNode sn
+                        = { TimeZoneSection, i - add, countRepeat(newFormat, i, 4), 0 };
                     newSectionNodes.append(sn);
                     appendSeparator(&newSeparators, newFormat, index, i - index, lastQuote);
                     i += sn.count - 1;
@@ -746,6 +727,36 @@ int QDateTimeParser::sectionMaxSize(int index) const
     return sectionMaxSize(sn.type, sn.count);
 }
 
+// Separator matching
+//
+// QTBUG-114909: user may be oblivious to difference between visibly
+// indistinguishable spacing characters. For now we only treat horizontal
+// spacing characters, excluding tab, as equivalent.
+
+static int matchesSeparator(QStringView text, QStringView separator)
+{
+    const auto isSimpleSpace = [](char32_t ch) {
+        // Distinguish tab, CR and the vertical spaces from the rest:
+        return ch == u' ' || (ch > 127 && QChar::isSpace(ch));
+    };
+    // -1 if not a match, else length of prefix of text that does match.
+    // First check for exact match
+    if (!text.startsWith(separator)) {
+        // Failing that, check for space-identifying match:
+        QStringIterator given(text), sep(separator);
+        while (sep.hasNext()) {
+            if (!given.hasNext())
+                return -1;
+            char32_t s = sep.next(), g = given.next();
+            if (s != g && !(isSimpleSpace(s) && isSimpleSpace(g)))
+                return -1;
+        }
+        // One side may have used a surrogate pair space where the other didn't:
+        return given.index();
+    }
+    return separator.size();
+}
+
 /*!
   \internal
 
@@ -829,7 +840,7 @@ QDateTimeParser::parseSection(const QDateTime &currentValue, int sectionIndex, i
     case TimeZoneSection:
         result = findTimeZone(sectionTextRef, defaultValue,
                               absoluteMax(sectionIndex),
-                              absoluteMin(sectionIndex));
+                              absoluteMin(sectionIndex), sn.count);
         break;
     case MonthSection:
     case DayOfWeekSectionShort:
@@ -864,12 +875,26 @@ QDateTimeParser::parseSection(const QDateTime &currentValue, int sectionIndex, i
     case MinuteSection:
     case SecondSection:
     case MSecSection: {
+        const auto checkSeparator = [&result, field=QStringView{m_text}.sliced(offset),
+                                     negativeYearOffset, sectionIndex, this]() {
+            // No-digit field if next separator is here, otherwise invalid.
+            const auto &sep = separators.at(sectionIndex + 1);
+            if (matchesSeparator(field.sliced(negativeYearOffset), sep) != -1)
+                result = ParsedSection(Intermediate, 0, negativeYearOffset);
+            else if (negativeYearOffset && matchesSeparator(field, sep) != -1)
+                result = ParsedSection(Intermediate, 0, 0);
+            else
+                return false;
+            return true;
+        };
         int used = negativeYearOffset;
-        // We already sliced off the - sign if it was legitimately present.
+        // We already sliced off the - sign if it was acceptable.
+        // QLocale::toUInt() would accept a sign, so we must reject it overtly:
         if (sectionTextRef.startsWith(u'-')
             || sectionTextRef.startsWith(u'+')) {
-            if (separators.at(sectionIndex + 1).startsWith(sectionTextRef[0]))
-                result = ParsedSection(Intermediate, 0, used);
+            // However, a sign here may indicate a field with no digits, if it
+            // starts the next separator:
+            checkSeparator();
             break;
         }
         QStringView digitsStr = sectionTextRef.left(digitCount(sectionTextRef));
@@ -881,7 +906,7 @@ QDateTimeParser::parseSection(const QDateTime &currentValue, int sectionIndex, i
             const int absMax = absoluteMax(sectionIndex);
             const int absMin = absoluteMin(sectionIndex);
 
-            int last = -1;
+            int lastVal = -1;
 
             for (; digitsStr.size(); digitsStr.chop(1)) {
                 bool ok = false;
@@ -897,52 +922,49 @@ QDateTimeParser::parseSection(const QDateTime &currentValue, int sectionIndex, i
                 }
 
                 QDTPDEBUG << digitsStr << value << digitsStr.size();
-                last = value;
+                lastVal = value;
                 used += digitsStr.size();
                 break;
             }
 
-            if (last == -1) {
-                const auto &sep = separators.at(sectionIndex + 1);
-                if (sep.startsWith(sectionTextRef[0])
-                    || (negate && sep.startsWith(m_text.at(offset))))
-                    result = ParsedSection(Intermediate, 0, 0);
-                else
+            if (lastVal == -1) {
+                if (!checkSeparator()) {
                     QDTPDEBUG << "invalid because" << sectionTextRef << "can't become a uint"
-                              << last;
+                              << lastVal;
+                }
             } else {
                 if (negate)
-                    last = -last;
+                    lastVal = -lastVal;
                 const FieldInfo fi = fieldInfo(sectionIndex);
                 const bool unfilled = used - negativeYearOffset < sectionmaxsize;
                 if (unfilled && fi & Fraction) { // typing 2 in a zzz field should be .200, not .002
                     for (int i = used; i < sectionmaxsize; ++i)
-                        last *= 10;
+                        lastVal *= 10;
                 }
                 // Even those *= 10s can't take last above absMax:
-                Q_ASSERT(negate ? last >= absMin : last <= absMax);
-                if (negate ? last > absMax : last < absMin) {
+                Q_ASSERT(negate ? lastVal >= absMin : lastVal <= absMax);
+                if (negate ? lastVal > absMax : lastVal < absMin) {
                     if (unfilled) {
-                        result = ParsedSection(Intermediate, last, used);
+                        result = ParsedSection(Intermediate, lastVal, used);
                     } else if (negate) {
-                        QDTPDEBUG << "invalid because" << last << "is greater than absoluteMax"
+                        QDTPDEBUG << "invalid because" << lastVal << "is greater than absoluteMax"
                                   << absMax;
                     } else {
-                        QDTPDEBUG << "invalid because" << last << "is less than absoluteMin"
+                        QDTPDEBUG << "invalid because" << lastVal << "is less than absoluteMin"
                                   << absMin;
                     }
 
                 } else if (unfilled && (fi & (FixedWidth | Numeric)) == (FixedWidth | Numeric)) {
                     if (skipToNextSection(sectionIndex, defaultValue, digitsStr)) {
                         const int missingZeroes = sectionmaxsize - digitsStr.size();
-                        result = ParsedSection(Acceptable, last, sectionmaxsize, missingZeroes);
+                        result = ParsedSection(Acceptable, lastVal, sectionmaxsize, missingZeroes);
                         m_text.insert(offset, QString(missingZeroes, u'0'));
                         ++(const_cast<QDateTimeParser*>(this)->sectionNodes[sectionIndex].zeroesAdded);
                     } else {
-                        result = ParsedSection(Intermediate, last, used);;
+                        result = ParsedSection(Intermediate, lastVal, used);;
                     }
                 } else {
-                    result = ParsedSection(Acceptable, last, used);
+                    result = ParsedSection(Acceptable, lastVal, used);
                 }
             }
         }
@@ -972,11 +994,7 @@ static int weekDayWithinMonth(QCalendar calendar, int year, int month, int day,
     const int maxDay = calendar.daysInMonth(month, year); // 0 if no such month
     day = maxDay > 1 ? qBound(1, day, maxDay) : qMax(1, day);
     day += dayOfWeekDiff(weekDay, calendar.dayOfWeek(QDate(year, month, day, calendar)));
-    if (day <= 0)
-        return day + 7;
-    if (maxDay > 0 && day > maxDay)
-        return day - 7;
-    return day;
+    return day <= 0 ? day + 7 : maxDay > 0 && day > maxDay ? day - 7 : day;
 }
 
 /*!
@@ -994,7 +1012,7 @@ static QDate actualDate(QDateTimeParser::Sections known, const QCalendar &calend
     if (actual.isValid() && year % 100 == year2digits && calendar.dayOfWeek(actual) == dayofweek)
         return actual; // The obvious candidate is fine :-)
 
-    if (dayofweek < 1 || dayofweek > 7) // Invalid: ignore
+    if (dayofweek < 1 || dayofweek > 7) // Intercallary (or invalid): ignore
         known &= ~QDateTimeParser::DayOfWeekSectionMask;
 
     // Assuming year > 0 ...
@@ -1018,16 +1036,21 @@ static QDate actualDate(QDateTimeParser::Sections known, const QCalendar &calend
     }
 
     QDate first(year, month, 1, calendar);
-    int last = known & QDateTimeParser::YearSection && known & QDateTimeParser::MonthSection
-        ? first.daysInMonth(calendar) : 0;
+    int last = known & QDateTimeParser::MonthSection
+        ? (known & QDateTimeParser::YearSection
+           ? calendar.daysInMonth(month, year) : calendar.daysInMonth(month))
+        : 0;
+    // We can only fix DOW if we know year as well as month (hence last):
+    const bool fixDayOfWeek = last && known & QDateTimeParser::YearSection
+            && known & QDateTimeParser::DayOfWeekSectionMask;
     // If we also know day-of-week, tweak last to the last in the month that matches it:
-    if (last && known & QDateTimeParser::DayOfWeekSectionMask) {
-        int diff = (dayofweek - calendar.dayOfWeek(first) - last) % 7;
+    if (fixDayOfWeek) {
+        const int diff = (dayofweek - calendar.dayOfWeek(first) - last) % 7;
         Q_ASSERT(diff <= 0); // C++11 specifies (-ve) % (+ve) to be <= 0.
         last += diff;
     }
     if (day < 1) {
-        if (known & QDateTimeParser::DayOfWeekSectionMask && last) {
+        if (fixDayOfWeek) {
             day = 1 + dayofweek - calendar.dayOfWeek(first);
             if (day < 1)
                 day += 7;
@@ -1035,7 +1058,7 @@ static QDate actualDate(QDateTimeParser::Sections known, const QCalendar &calend
             day = 1;
         }
         known &= ~QDateTimeParser::DaySection;
-    } else if (day > 31) {
+    } else if (day > calendar.maximumDaysInMonth()) {
         day = last;
         known &= ~QDateTimeParser::DaySection;
     } else if (last && day > last && (known & QDateTimeParser::DaySection) == 0) {
@@ -1157,6 +1180,24 @@ static QTime actualTime(QDateTimeParser::Sections known,
     return actual;
 }
 
+/*
+  \internal
+*/
+static int startsWithLocalTimeZone(QStringView name, const QDateTime &when)
+{
+    // On MS-Win, at least when system zone is UTC, the tzname[]s may be empty.
+    for (int i = 0; i < 2; ++i) {
+        const QString zone(qTzName(i));
+        if (!zone.isEmpty() && name.startsWith(zone))
+            return zone.size();
+    }
+    // Mimic what QLocale::toString() would have used, to ensure round-trips work:
+    const QString local = QDateTime(when.date(), when.time()).timeZoneAbbreviation();
+    if (name.startsWith(local))
+        return local.size();
+    return 0;
+}
+
 /*!
   \internal
 */
@@ -1179,26 +1220,7 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
     int second = defaultTime.second();
     int msec = defaultTime.msec();
     int dayofweek = calendar.dayOfWeek(defaultDate);
-    Qt::TimeSpec tspec = defaultValue.timeSpec();
-    int zoneOffset = 0; // In seconds; local - UTC
-#if QT_CONFIG(timezone)
-    QTimeZone timeZone;
-#endif
-    switch (tspec) {
-    case Qt::OffsetFromUTC: // timeZone is ignored
-        zoneOffset = defaultValue.offsetFromUtc();
-        break;
-#if QT_CONFIG(timezone)
-    case Qt::TimeZone:
-        timeZone = defaultValue.timeZone();
-        if (timeZone.isValid())
-            zoneOffset = timeZone.offsetFromUtc(defaultValue);
-        // else: is there anything we can do about this ?
-        break;
-#endif
-    default: // zoneOffset and timeZone are ignored
-        break;
-    }
+    QTimeZone timeZone = defaultValue.timeRepresentation();
 
     int ampm = -1;
     Sections isSet = NoSection;
@@ -1206,29 +1228,25 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
     for (int index = 0; index < sectionNodesCount; ++index) {
         Q_ASSERT(state != Invalid);
         const QString &separator = separators.at(index);
-        if (QStringView{m_text}.mid(pos, separator.size()) != separator) {
-            QDTPDEBUG << "invalid because" << QStringView{m_text}.mid(pos, separator.size())
-                      << "!=" << separator
+        int step = matchesSeparator(QStringView{m_text}.sliced(pos), separator);
+        if (step == -1) {
+            QDTPDEBUG << "invalid because" << QStringView{m_text}.sliced(pos)
+                      << "does not start with" << separator
                       << index << pos << currentSectionIndex;
             return StateNode();
         }
-        pos += separator.size();
+        pos += step;
         sectionNodes[index].pos = pos;
         int *current = nullptr;
+        int zoneOffset; // Needed to serve as *current when setting zone
         const SectionNode sn = sectionNodes.at(index);
-        ParsedSection sect;
-
-        {
+        const QDateTime usedDateTime = [&] {
             const QDate date = actualDate(isSet, calendar, year, year2digits,
                                           month, day, dayofweek);
             const QTime time = actualTime(isSet, hour, hour12, ampm, minute, second, msec);
-            sect = parseSection(
-#if QT_CONFIG(timezone)
-                                tspec == Qt::TimeZone ? QDateTime(date, time, timeZone) :
-#endif
-                                QDateTime(date, time, tspec, zoneOffset),
-                                index, pos);
-        }
+            return QDateTime(date, time, timeZone);
+        }();
+        ParsedSection sect = parseSection(usedDateTime, index, pos);
 
         QDTPDEBUG << "sectionValue" << sn.name() << m_text
                   << "pos" << pos << "used" << sect.used << stateName(sect.state);
@@ -1262,16 +1280,15 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
                 const bool isUtc = zoneName == "Z"_L1 || zoneName == "UTC"_L1;
 
                 if (isUtc || isUtcOffset) {
-                    tspec = sect.value ? Qt::OffsetFromUTC : Qt::UTC;
-                } else {
+                    timeZone = QTimeZone::fromSecondsAheadOfUtc(sect.value);
 #if QT_CONFIG(timezone)
-                    timeZone = QTimeZone(zoneName.toLatin1());
-                    tspec = timeZone.isValid()
-                        ? Qt::TimeZone
-                        : (Q_ASSERT(startsWithLocalTimeZone(zoneName)), Qt::LocalTime);
-#else
-                    tspec = Qt::LocalTime;
+                } else if (startsWithLocalTimeZone(zoneName, usedDateTime) != sect.used) {
+                    QTimeZone namedZone = QTimeZone(zoneName.toLatin1());
+                    Q_ASSERT(namedZone.isValid());
+                    timeZone = namedZone;
 #endif
+                } else {
+                    timeZone = QTimeZone::LocalTime;
                 }
             }
             break;
@@ -1312,9 +1329,10 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
         isSet |= sn.type;
     }
 
-    if (QStringView{m_text}.sliced(pos) != separators.last()) {
+    int step = matchesSeparator(QStringView{m_text}.sliced(pos), separators.last());
+    if (step == -1 || step + pos < m_text.size()) {
         QDTPDEBUG << "invalid because" << QStringView{m_text}.sliced(pos)
-                  << "!=" << separators.last() << pos;
+                  << "does not match" << separators.last() << pos;
         return StateNode();
     }
 
@@ -1412,11 +1430,7 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
 
     const QDate date(year, month, day, calendar);
     const QTime time(hour, minute, second, msec);
-    const QDateTime when =
-#if QT_CONFIG(timezone)
-            tspec == Qt::TimeZone ? QDateTime(date, time, timeZone) :
-#endif
-            QDateTime(date, time, tspec, zoneOffset);
+    const QDateTime when = QDateTime(date, time, timeZone);
 
     // If hour wasn't specified, check the default we're using exists on the
     // given date (which might be a spring-forward, skipping an hour).
@@ -1425,11 +1439,7 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
         case QMetaType::QDateTime: {
             qint64 msecs = when.toMSecsSinceEpoch();
             // Fortunately, that gets a useful answer, even though when is invalid ...
-            const QDateTime replace =
-#if QT_CONFIG(timezone)
-                tspec == Qt::TimeZone ? QDateTime::fromMSecsSinceEpoch(msecs, timeZone) :
-#endif
-                QDateTime::fromMSecsSinceEpoch(msecs, tspec, zoneOffset);
+            const QDateTime replace = QDateTime::fromMSecsSinceEpoch(msecs, timeZone);
             const QTime tick = replace.time();
             if (replace.date() == date
                 && (!(isSet & MinuteSection) || tick.minute() == minute)
@@ -1440,14 +1450,13 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
         } break;
         case QMetaType::QDate:
             // Don't care about time, so just use start of day (and ignore spec):
-            return StateNode(date.startOfDay(Qt::UTC), state, padding, conflicts);
+            return StateNode(date.startOfDay(QTimeZone::UTC), state, padding, conflicts);
             break;
         case QMetaType::QTime:
-            // Don't care about date or spec, so pick a safe spec:
-            return StateNode(QDateTime(date, time, Qt::UTC), state, padding, conflicts);
+            // Don't care about date or representation, so pick a safe representation:
+            return StateNode(QDateTime(date, time, QTimeZone::UTC), state, padding, conflicts);
         default:
-            Q_UNREACHABLE();
-            return StateNode();
+            Q_UNREACHABLE_RETURN(StateNode());
         }
     }
 
@@ -1700,12 +1709,17 @@ int QDateTimeParser::findDay(const QString &str1, int startDay, int sectionIndex
 
   Return's .value is UTC offset in seconds.
   The caller must verify that the offset is within a valid range.
+  The mode is 1 for permissive parsing, 2 and 3 for strict offset-only format
+  (no UTC prefix) with no colon for 2 and a colon for 3.
  */
-QDateTimeParser::ParsedSection QDateTimeParser::findUtcOffset(QStringView str) const
+QDateTimeParser::ParsedSection QDateTimeParser::findUtcOffset(QStringView str, int mode) const
 {
+    Q_ASSERT(mode > 0 && mode < 4);
     const bool startsWithUtc = str.startsWith("UTC"_L1);
-    // Get rid of UTC prefix if it exists
+    // Deal with UTC prefix if present:
     if (startsWithUtc) {
+        if (mode != 1)
+            return ParsedSection();
         str = str.sliced(3);
         if (str.isEmpty())
             return ParsedSection(Acceptable, 0, 3);
@@ -1739,6 +1753,8 @@ QDateTimeParser::ParsedSection QDateTimeParser::findUtcOffset(QStringView str) c
         i = hoursLength;
         hasColon = false;
     }
+    if (mode == (hasColon ? 2 : 3))
+        return ParsedSection();
     str.truncate(i);  // The rest of the string is not part of the UTC offset
 
     bool isInt = false;
@@ -1777,7 +1793,7 @@ QDateTimeParser::ParsedSection QDateTimeParser::findUtcOffset(QStringView str) c
 QDateTimeParser::ParsedSection
 QDateTimeParser::findTimeZoneName(QStringView str, const QDateTime &when) const
 {
-    const int systemLength = startsWithLocalTimeZone(str);
+    const int systemLength = startsWithLocalTimeZone(str, when);
 #if QT_CONFIG(timezone)
     // Collect up plausibly-valid characters; let QTimeZone work out what's
     // truly valid.
@@ -1823,17 +1839,26 @@ QDateTimeParser::findTimeZoneName(QStringView str, const QDateTime &when) const
 
   Return's .value is zone's offset, zone time - UTC time, in seconds.
   See QTimeZonePrivate::isValidId() for the format of zone names.
- */
+
+  The mode is the number of 't' characters in the field specifier:
+  * 1: any recognized format
+  * 2: only the simple offset format, without colon
+  * 3: only the simple offset format, with colon
+  * 4: only a zone name
+*/
 QDateTimeParser::ParsedSection
 QDateTimeParser::findTimeZone(QStringView str, const QDateTime &when,
-                              int maxVal, int minVal) const
+                              int maxVal, int minVal, int mode) const
 {
+    Q_ASSERT(mode > 0 && mode <= 4);
     // Short-cut Zulu suffix when it's all there is (rather than a prefix match):
-    if (str == u'Z')
+    if (mode == 1 && str == u'Z')
         return ParsedSection(Acceptable, 0, 1);
 
-    ParsedSection section = findUtcOffset(str);
-    if (section.used <= 0)  // if nothing used, try time zone parsing
+    ParsedSection section;
+    if (mode != 4)
+        section = findUtcOffset(str, mode);
+    if (mode != 2 && mode != 3 && section.used <= 0)  // if nothing used, try time zone parsing
         section = findTimeZoneName(str, when);
     // It can be a well formed time zone specifier, but with value out of range
     if (section.state == Acceptable && (section.value < minVal || section.value > maxVal))
@@ -1841,11 +1866,13 @@ QDateTimeParser::findTimeZone(QStringView str, const QDateTime &when,
     if (section.used > 0)
         return section;
 
-    // Check if string is UTC or alias to UTC, after all other options
-    if (str.startsWith("UTC"_L1))
-        return ParsedSection(Acceptable, 0, 3);
-    if (str.startsWith(u'Z'))
-        return ParsedSection(Acceptable, 0, 1);
+    if (mode == 1) {
+        // Check if string is UTC or alias to UTC, after all other options
+        if (str.startsWith("UTC"_L1))
+            return ParsedSection(Acceptable, 0, 3);
+        if (str.startsWith(u'Z'))
+            return ParsedSection(Acceptable, 0, 1);
+    }
 
     return ParsedSection();
 }
@@ -1904,20 +1931,21 @@ QDateTimeParser::AmPmFinder QDateTimeParser::findAmPm(QString &str, int sectionI
 
     bool broken[2] = {false, false};
     for (int i=0; i<size; ++i) {
-        if (str.at(i) != space) {
+        const QChar ch = str.at(i);
+        if (ch != space) {
             for (int j=0; j<2; ++j) {
                 if (!broken[j]) {
-                    int index = ampm[j].indexOf(str.at(i));
-                    QDTPDEBUG << "looking for" << str.at(i)
+                    int index = ampm[j].indexOf(ch);
+                    QDTPDEBUG << "looking for" << ch
                               << "in" << ampm[j] << "and got" << index;
                     if (index == -1) {
-                        if (str.at(i).category() == QChar::Letter_Uppercase) {
-                            index = ampm[j].indexOf(str.at(i).toLower());
-                            QDTPDEBUG << "trying with" << str.at(i).toLower()
+                        if (ch.category() == QChar::Letter_Uppercase) {
+                            index = ampm[j].indexOf(ch.toLower());
+                            QDTPDEBUG << "trying with" << ch.toLower()
                                       << "in" << ampm[j] << "and got" << index;
-                        } else if (str.at(i).category() == QChar::Letter_Lowercase) {
-                            index = ampm[j].indexOf(str.at(i).toUpper());
-                            QDTPDEBUG << "trying with" << str.at(i).toUpper()
+                        } else if (ch.category() == QChar::Letter_Lowercase) {
+                            index = ampm[j].indexOf(ch.toUpper());
+                            QDTPDEBUG << "trying with" << ch.toUpper()
                                       << "in" << ampm[j] << "and got" << index;
                         }
                         if (index == -1) {
@@ -2137,22 +2165,22 @@ bool QDateTimeParser::skipToNextSection(int index, const QDateTime &current, QSt
 QString QDateTimeParser::SectionNode::name(QDateTimeParser::Section s)
 {
     switch (s) {
-    case QDateTimeParser::AmPmSection: return "AmPmSection"_L1;
-    case QDateTimeParser::DaySection: return "DaySection"_L1;
-    case QDateTimeParser::DayOfWeekSectionShort: return "DayOfWeekSectionShort"_L1;
-    case QDateTimeParser::DayOfWeekSectionLong: return "DayOfWeekSectionLong"_L1;
-    case QDateTimeParser::Hour24Section: return "Hour24Section"_L1;
-    case QDateTimeParser::Hour12Section: return "Hour12Section"_L1;
-    case QDateTimeParser::MSecSection: return "MSecSection"_L1;
-    case QDateTimeParser::MinuteSection: return "MinuteSection"_L1;
-    case QDateTimeParser::MonthSection: return "MonthSection"_L1;
-    case QDateTimeParser::SecondSection: return "SecondSection"_L1;
-    case QDateTimeParser::TimeZoneSection: return "TimeZoneSection"_L1;
-    case QDateTimeParser::YearSection: return "YearSection"_L1;
-    case QDateTimeParser::YearSection2Digits: return "YearSection2Digits"_L1;
-    case QDateTimeParser::NoSection: return "NoSection"_L1;
-    case QDateTimeParser::FirstSection: return "FirstSection"_L1;
-    case QDateTimeParser::LastSection: return "LastSection"_L1;
+    case AmPmSection: return "AmPmSection"_L1;
+    case DaySection: return "DaySection"_L1;
+    case DayOfWeekSectionShort: return "DayOfWeekSectionShort"_L1;
+    case DayOfWeekSectionLong: return "DayOfWeekSectionLong"_L1;
+    case Hour24Section: return "Hour24Section"_L1;
+    case Hour12Section: return "Hour12Section"_L1;
+    case MSecSection: return "MSecSection"_L1;
+    case MinuteSection: return "MinuteSection"_L1;
+    case MonthSection: return "MonthSection"_L1;
+    case SecondSection: return "SecondSection"_L1;
+    case TimeZoneSection: return "TimeZoneSection"_L1;
+    case YearSection: return "YearSection"_L1;
+    case YearSection2Digits: return "YearSection2Digits"_L1;
+    case NoSection: return "NoSection"_L1;
+    case FirstSection: return "FirstSection"_L1;
+    case LastSection: return "LastSection"_L1;
     default: return "Unknown section "_L1 + QString::number(int(s));
     }
 }
@@ -2175,7 +2203,7 @@ QString QDateTimeParser::stateName(State s) const
 // Only called when we want only one of date or time; use UTC to avoid bogus DST issues.
 bool QDateTimeParser::fromString(const QString &t, QDate *date, QTime *time) const
 {
-    QDateTime val(QDate(1900, 1, 1).startOfDay(Qt::UTC));
+    QDateTime val(QDate(1900, 1, 1).startOfDay(QTimeZone::UTC));
     const StateNode tmp = parse(t, -1, val, false);
     if (tmp.state != Acceptable || tmp.conflicts)
         return false;
@@ -2201,8 +2229,8 @@ bool QDateTimeParser::fromString(const QString &t, QDate *date, QTime *time) con
 // Only called when we want both date and time; default to local time.
 bool QDateTimeParser::fromString(const QString &t, QDateTime *datetime) const
 {
-    QDateTime val(QDate(1900, 1, 1).startOfDay());
-    const StateNode tmp = parse(t, -1, val, false);
+    static const QDateTime defaultLocalTime = QDate(1900, 1, 1).startOfDay();
+    const StateNode tmp = parse(t, -1, defaultLocalTime, false);
     if (datetime)
         *datetime = tmp.value;
     return tmp.state == Acceptable && !tmp.conflicts && tmp.value.isValid();
@@ -2215,7 +2243,7 @@ QDateTime QDateTimeParser::getMinimum() const
     //     method. At the time of writing, this is done by QDateTimeEditPrivate.
 
     // Cache the only case
-    static const QDateTime localTimeMin(QDATETIMEEDIT_DATE_MIN.startOfDay(Qt::LocalTime));
+    static const QDateTime localTimeMin(QDATETIMEEDIT_DATE_MIN.startOfDay());
     return localTimeMin;
 }
 
@@ -2226,7 +2254,7 @@ QDateTime QDateTimeParser::getMaximum() const
     //     method. At the time of writing, this is done by QDateTimeEditPrivate.
 
     // Cache the only case
-    static const QDateTime localTimeMax(QDATETIMEEDIT_DATE_MAX.endOfDay(Qt::LocalTime));
+    static const QDateTime localTimeMax(QDATETIMEEDIT_DATE_MAX.endOfDay());
     return localTimeMax;
 }
 
@@ -2240,8 +2268,7 @@ QString QDateTimeParser::getAmPmText(AmPm ap, Case cs) const
     case LowerCase: return raw.toLower();
     case NativeCase: return raw;
     }
-    Q_UNREACHABLE();
-    return raw;
+    Q_UNREACHABLE_RETURN(raw);
 }
 
 /*
