@@ -30,14 +30,7 @@
 #define _int64 __int64
 #endif
 
-
 #include <oci.h>
-#ifdef max
-#undef max
-#endif
-#ifdef min
-#undef min
-#endif
 
 #include <stdlib.h>
 
@@ -170,7 +163,7 @@ QDateTime QOCIDateTime::fromOCIDateTime(OCIEnv *env, OCIError *err, OCIDateTime
         secondsOffset = -secondsOffset;
     // OCIDateTimeGetTime gives "fractions of second" as nanoseconds
     return QDateTime(QDate(year, month, day), QTime(hour, minute, second, nsec / 1000000),
-                     Qt::OffsetFromUTC, secondsOffset);
+                     QTimeZone::fromSecondsAheadOfUtc(secondsOffset));
 }
 
 struct TempStorage {
@@ -196,6 +189,7 @@ public:
     OCISession *authp = nullptr;
     OCITrans *trans = nullptr;
     OCIError *err = nullptr;
+    ub4 authMode = OCI_DEFAULT;
     bool transaction = false;
     int serverVersion = -1;
     int prefetchRows = -1;
@@ -2193,17 +2187,16 @@ bool QOCIDriver::hasFeature(DriverFeature f) const
 
 static void qParseOpts(const QString &options, QOCIDriverPrivate *d)
 {
-    const QStringList opts(options.split(u';', Qt::SkipEmptyParts));
-    for (int i = 0; i < opts.count(); ++i) {
-        const QString tmp(opts.at(i));
+    const QVector<QStringView> opts(QStringView(options).split(u';', Qt::SkipEmptyParts));
+    for (const auto tmp : opts) {
         qsizetype idx;
         if ((idx = tmp.indexOf(u'=')) == -1) {
             qWarning("QOCIDriver::parseArgs: Invalid parameter: '%s'",
                      tmp.toLocal8Bit().constData());
             continue;
         }
-        const QString opt = tmp.left(idx);
-        const QString val = tmp.mid(idx + 1).simplified();
+        const QStringView opt = tmp.left(idx);
+        const QStringView val = tmp.mid(idx + 1).trimmed();
         bool ok;
         if (opt == "OCI_ATTR_PREFETCH_ROWS"_L1) {
             d->prefetchRows = val.toInt(&ok);
@@ -2213,9 +2206,18 @@ static void qParseOpts(const QString &options, QOCIDriverPrivate *d)
             d->prefetchMem = val.toInt(&ok);
             if (!ok)
                 d->prefetchMem = -1;
+        } else if (opt == "OCI_AUTH_MODE"_L1) {
+            if (val == "OCI_SYSDBA"_L1) {
+                d->authMode = OCI_SYSDBA;
+            } else if (val == "OCI_SYSOPER"_L1) {
+                d->authMode = OCI_SYSOPER;
+            } else if (val != "OCI_DEFAULT"_L1) {
+                qWarning("QOCIDriver::parseArgs: Unsupported value for OCI_AUTH_MODE: '%s'",
+                         val.toLocal8Bit().constData());
+            }
         } else {
-            qWarning ("QOCIDriver::parseArgs: Invalid parameter: '%s'",
-                      opt.toLocal8Bit().constData());
+            qWarning("QOCIDriver::parseArgs: Invalid parameter: '%s'",
+                     opt.toLocal8Bit().constData());
         }
     }
 }
@@ -2279,9 +2281,9 @@ bool QOCIDriver::open(const QString & db,
 
     if (r == OCI_SUCCESS) {
         if (user.isEmpty() && password.isEmpty())
-            r = OCISessionBegin(d->svc, d->err, d->authp, OCI_CRED_EXT, OCI_DEFAULT);
+            r = OCISessionBegin(d->svc, d->err, d->authp, OCI_CRED_EXT, d->authMode);
         else
-            r = OCISessionBegin(d->svc, d->err, d->authp, OCI_CRED_RDBMS, OCI_DEFAULT);
+            r = OCISessionBegin(d->svc, d->err, d->authp, OCI_CRED_RDBMS, d->authMode);
     }
     if (r == OCI_SUCCESS || r == OCI_SUCCESS_WITH_INFO)
         r = OCIAttrSet(d->svc, OCI_HTYPE_SVCCTX, d->authp, 0, OCI_ATTR_SESSION, d->err);
@@ -2744,10 +2746,19 @@ QString QOCIDriver::escapeIdentifier(const QString &identifier, IdentifierType t
     QString res = identifier;
     if (!identifier.isEmpty() && !isIdentifierEscaped(identifier, type)) {
         res.replace(u'"', "\"\""_L1);
-        res.prepend(u'"').append(u'"');
         res.replace(u'.', "\".\""_L1);
+        res = u'"' + res + u'"';
     }
     return res;
 }
 
+int QOCIDriver::maximumIdentifierLength(IdentifierType type) const
+{
+    Q_D(const QOCIDriver);
+    Q_UNUSED(type);
+    return d->serverVersion > 12 ? 128 : 30;
+}
+
 QT_END_NAMESPACE
+
+#include "moc_qsql_oci_p.cpp"
