@@ -6,16 +6,14 @@
 #include "qdebug.h"
 #include "qcoreapplication.h"
 #include "qreadwritelock.h"
-#include "qsqlresult.h"
 #include "qsqldriver.h"
 #include "qsqldriverplugin.h"
 #include "qsqlindex.h"
+#include "QtCore/qapplicationstatic.h"
 #include "private/qfactoryloader_p.h"
 #include "private/qsqlnulldriver_p.h"
-#include "qmutex.h"
 #include "qhash.h"
 #include "qthread.h"
-#include <stdlib.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -24,34 +22,39 @@ using namespace Qt::StringLiterals;
 Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
                           (QSqlDriverFactoryInterface_iid, "/sqldrivers"_L1))
 
-const char *QSqlDatabase::defaultConnection = const_cast<char *>("qt_sql_default_connection");
+const char *QSqlDatabase::defaultConnection = "qt_sql_default_connection";
 
-typedef QHash<QString, QSqlDriverCreatorBase*> DriverDict;
-
-class QConnectionDict: public QHash<QString, QSqlDatabase>
-{
-public:
-    inline bool contains_ts(const QString &key)
+namespace {
+    struct QtSqlGlobals
     {
-        QReadLocker locker(&lock);
-        return contains(key);
-    }
-    inline QStringList keys_ts() const
-    {
-        QReadLocker locker(&lock);
-        return keys();
-    }
-
-    mutable QReadWriteLock lock;
-};
-Q_GLOBAL_STATIC(QConnectionDict, dbDict)
+        ~QtSqlGlobals();
+        QSqlDatabase connection(const QString &key) const
+        {
+          QReadLocker locker(&lock);
+          return connections.value(key);
+        }
+        bool connectionExists(const QString &key) const
+        {
+            QReadLocker locker(&lock);
+            return connections.contains(key);
+        }
+        QStringList connectionNames() const
+        {
+            QReadLocker locker(&lock);
+            return connections.keys();
+        }
+        mutable QReadWriteLock lock;
+        QHash<QString, QSqlDriverCreatorBase*> registeredDrivers;
+        QHash<QString, QSqlDatabase> connections;
+    };
+}
+Q_APPLICATION_STATIC(QtSqlGlobals, s_sqlGlobals)
 
 class QSqlDatabasePrivate
 {
 public:
-    QSqlDatabasePrivate(QSqlDatabase *d, QSqlDriver *dr = nullptr):
+    QSqlDatabasePrivate(QSqlDriver *dr):
         ref(1),
-        q(d),
         driver(dr),
         port(-1)
     {
@@ -64,7 +67,6 @@ public:
     void disable();
 
     QAtomicInt ref;
-    QSqlDatabase *q;
     QSqlDriver* driver;
     QString dbname;
     QString uname;
@@ -81,13 +83,10 @@ public:
     static void addDatabase(const QSqlDatabase &db, const QString & name);
     static void removeDatabase(const QString& name);
     static void invalidateDb(const QSqlDatabase &db, const QString &name, bool doWarn = true);
-    static DriverDict &driverDict();
-    static void cleanConnections();
 };
 
 QSqlDatabasePrivate::QSqlDatabasePrivate(const QSqlDatabasePrivate &other) : ref(1)
 {
-    q = other.q;
     dbname = other.dbname;
     uname = other.uname;
     pword = other.pword;
@@ -107,43 +106,17 @@ QSqlDatabasePrivate::~QSqlDatabasePrivate()
         delete driver;
 }
 
-void QSqlDatabasePrivate::cleanConnections()
+QtSqlGlobals::~QtSqlGlobals()
 {
-    QConnectionDict *dict = dbDict();
-    Q_ASSERT(dict);
-    QWriteLocker locker(&dict->lock);
-
-    QConnectionDict::iterator it = dict->begin();
-    while (it != dict->end()) {
-        invalidateDb(it.value(), it.key(), false);
-        ++it;
-    }
-    dict->clear();
-}
-
-static bool qDriverDictInit = false;
-static void cleanDriverDict()
-{
-    qDeleteAll(QSqlDatabasePrivate::driverDict());
-    QSqlDatabasePrivate::driverDict().clear();
-    QSqlDatabasePrivate::cleanConnections();
-    qDriverDictInit = false;
-}
-
-DriverDict &QSqlDatabasePrivate::driverDict()
-{
-    static DriverDict dict;
-    if (!qDriverDictInit) {
-        qDriverDictInit = true;
-        qAddPostRoutine(cleanDriverDict);
-    }
-    return dict;
+    qDeleteAll(registeredDrivers);
+    for (const auto &[k, v] : connections.asKeyValueRange())
+        QSqlDatabasePrivate::invalidateDb(v, k, false);
 }
 
 QSqlDatabasePrivate *QSqlDatabasePrivate::shared_null()
 {
     static QSqlNullDriver dr;
-    static QSqlDatabasePrivate n(nullptr, &dr);
+    static QSqlDatabasePrivate n(&dr);
     return &n;
 }
 
@@ -159,28 +132,26 @@ void QSqlDatabasePrivate::invalidateDb(const QSqlDatabase &db, const QString &na
 
 void QSqlDatabasePrivate::removeDatabase(const QString &name)
 {
-    QConnectionDict *dict = dbDict();
-    Q_ASSERT(dict);
-    QWriteLocker locker(&dict->lock);
+    QtSqlGlobals *sqlGlobals = s_sqlGlobals();
+    QWriteLocker locker(&sqlGlobals->lock);
 
-    if (!dict->contains(name))
+    if (!sqlGlobals->connections.contains(name))
         return;
 
-    invalidateDb(dict->take(name), name);
+    invalidateDb(sqlGlobals->connections.take(name), name);
 }
 
 void QSqlDatabasePrivate::addDatabase(const QSqlDatabase &db, const QString &name)
 {
-    QConnectionDict *dict = dbDict();
-    Q_ASSERT(dict);
-    QWriteLocker locker(&dict->lock);
+    QtSqlGlobals *sqlGlobals = s_sqlGlobals();
+    QWriteLocker locker(&sqlGlobals->lock);
 
-    if (dict->contains(name)) {
-        invalidateDb(dict->take(name), name);
+    if (sqlGlobals->connections.contains(name)) {
+        invalidateDb(sqlGlobals->connections.take(name), name);
         qWarning("QSqlDatabasePrivate::addDatabase: duplicate connection name '%s', old "
                  "connection removed.", name.toLocal8Bit().data());
     }
-    dict->insert(name, db);
+    sqlGlobals->connections.insert(name, db);
     db.d->connName = name;
 }
 
@@ -188,12 +159,7 @@ void QSqlDatabasePrivate::addDatabase(const QSqlDatabase &db, const QString &nam
 */
 QSqlDatabase QSqlDatabasePrivate::database(const QString& name, bool open)
 {
-    const QConnectionDict *dict = dbDict();
-    Q_ASSERT(dict);
-
-    dict->lock.lockForRead();
-    QSqlDatabase db = dict->value(name);
-    dict->lock.unlock();
+    QSqlDatabase db = s_sqlGlobals()->connection(name);
     if (!db.isValid())
         return db;
     if (db.driver()->thread() != QThread::currentThread()) {
@@ -215,7 +181,6 @@ QSqlDatabase QSqlDatabasePrivate::database(const QString& name, bool open)
 */
 void QSqlDatabasePrivate::copy(const QSqlDatabasePrivate *other)
 {
-    q = other->q;
     dbname = other->dbname;
     uname = other->uname;
     pword = other->pword;
@@ -392,9 +357,6 @@ void QSqlDatabasePrivate::disable()
         \li registers a custom-made driver
     \endtable
 
-    \note QSqlDatabase::exec() is deprecated. Use QSqlQuery::exec()
-    instead.
-
     \note When using transactions, you must start the
     transaction before you create your query.
 
@@ -502,19 +464,20 @@ QStringList QSqlDatabase::drivers()
 
     if (QFactoryLoader *fl = loader()) {
         typedef QMultiMap<int, QString> PluginKeyMap;
-        typedef PluginKeyMap::const_iterator PluginKeyMapConstIterator;
 
         const PluginKeyMap keyMap = fl->keyMap();
-        const PluginKeyMapConstIterator cend = keyMap.constEnd();
-        for (PluginKeyMapConstIterator it = keyMap.constBegin(); it != cend; ++it)
-            if (!list.contains(it.value()))
-                list << it.value();
+        for (const QString &val : keyMap) {
+            if (!list.contains(val))
+                list << val;
+        }
     }
 
-    DriverDict dict = QSqlDatabasePrivate::driverDict();
-    for (DriverDict::const_iterator i = dict.constBegin(); i != dict.constEnd(); ++i) {
-        if (!list.contains(i.key()))
-            list << i.key();
+    QtSqlGlobals *sqlGlobals = s_sqlGlobals();
+    QReadLocker locker(&sqlGlobals->lock);
+    const auto &dict = sqlGlobals->registeredDrivers;
+    for (const auto &[k, _] : dict.asKeyValueRange()) {
+        if (!list.contains(k))
+            list << k;
     }
 
     return list;
@@ -535,9 +498,11 @@ QStringList QSqlDatabase::drivers()
 */
 void QSqlDatabase::registerSqlDriver(const QString& name, QSqlDriverCreatorBase *creator)
 {
-    delete QSqlDatabasePrivate::driverDict().take(name);
+    QtSqlGlobals *sqlGlobals = s_sqlGlobals();
+    QWriteLocker locker(&sqlGlobals->lock);
+    delete sqlGlobals->registeredDrivers.take(name);
     if (creator)
-        QSqlDatabasePrivate::driverDict().insert(name, creator);
+        sqlGlobals->registeredDrivers.insert(name, creator);
 }
 
 /*!
@@ -551,7 +516,7 @@ void QSqlDatabase::registerSqlDriver(const QString& name, QSqlDriverCreatorBase
 
 bool QSqlDatabase::contains(const QString& connectionName)
 {
-    return dbDict()->contains_ts(connectionName);
+    return s_sqlGlobals()->connectionExists(connectionName);
 }
 
 /*!
@@ -563,7 +528,7 @@ bool QSqlDatabase::contains(const QString& connectionName)
 */
 QStringList QSqlDatabase::connectionNames()
 {
-    return dbDict()->keys_ts();
+    return s_sqlGlobals()->connectionNames();
 }
 
 /*!
@@ -584,6 +549,7 @@ QStringList QSqlDatabase::connectionNames()
     \row \li QODBC    \li ODBC Driver (includes Microsoft SQL Server)
     \row \li QPSQL    \li PostgreSQL Driver
     \row \li QSQLITE  \li SQLite version 3 or above
+    \row \li QMIMER  \li Mimer SQL 11 or above
     \endtable
 
     Additional third party drivers, including your own custom
@@ -593,8 +559,8 @@ QStringList QSqlDatabase::connectionNames()
 */
 
 QSqlDatabase::QSqlDatabase(const QString &type)
+   : d(new QSqlDatabasePrivate(nullptr))
 {
-    d = new QSqlDatabasePrivate(this);
     d->init(type);
 }
 
@@ -605,8 +571,8 @@ QSqlDatabase::QSqlDatabase(const QString &type)
 */
 
 QSqlDatabase::QSqlDatabase(QSqlDriver *driver)
+    : d(new QSqlDatabasePrivate(driver))
 {
-    d = new QSqlDatabasePrivate(this, driver);
 }
 
 /*!
@@ -615,8 +581,8 @@ QSqlDatabase::QSqlDatabase(QSqlDriver *driver)
     objects.
 */
 QSqlDatabase::QSqlDatabase()
+    : d(QSqlDatabasePrivate::shared_null())
 {
-    d = QSqlDatabasePrivate::shared_null();
     d->ref.ref();
 }
 
@@ -649,13 +615,12 @@ void QSqlDatabasePrivate::init(const QString &type)
     drvName = type;
 
     if (!driver) {
-        DriverDict dict = QSqlDatabasePrivate::driverDict();
-        for (DriverDict::const_iterator it = dict.constBegin();
-             it != dict.constEnd() && !driver; ++it) {
-            if (type == it.key()) {
-                driver = ((QSqlDriverCreatorBase*)(*it))->createObject();
-            }
-        }
+        QtSqlGlobals *sqlGlobals = s_sqlGlobals();
+        QReadLocker locker(&sqlGlobals->lock);
+        const auto &dict = sqlGlobals->registeredDrivers;
+        auto it = dict.find(type);
+        if (it != dict.end())
+            driver = it.value()->createObject();
     }
 
     if (!driver && loader())
@@ -695,8 +660,9 @@ QSqlDatabase::~QSqlDatabase()
     lastError() is not affected.
 
     \sa QSqlQuery, lastError()
+    \deprecated [6.6] Use QSqlQuery::exec() instead.
 */
-
+#if QT_DEPRECATED_SINCE(6, 6)
 QSqlQuery QSqlDatabase::exec(const QString & query) const
 {
     QSqlQuery r(d->driver->createResult());
@@ -706,6 +672,7 @@ QSqlQuery QSqlDatabase::exec(const QString & query) const
     }
     return r;
 }
+#endif
 
 /*!
     Opens the database connection using the current connection
@@ -1099,98 +1066,8 @@ QSqlRecord QSqlDatabase::record(const QString& tablename) const
 
     The format of the \a options string is a semicolon separated list
     of option names or option=value pairs. The options depend on the
-    database client used:
-
-    \table
-    \header \li ODBC \li MySQL \li PostgreSQL
-    \row
-
-    \li
-    \list
-    \li SQL_ATTR_ACCESS_MODE
-    \li SQL_ATTR_LOGIN_TIMEOUT
-    \li SQL_ATTR_CONNECTION_TIMEOUT
-    \li SQL_ATTR_CURRENT_CATALOG
-    \li SQL_ATTR_METADATA_ID
-    \li SQL_ATTR_PACKET_SIZE
-    \li SQL_ATTR_TRACEFILE
-    \li SQL_ATTR_TRACE
-    \li SQL_ATTR_CONNECTION_POOLING
-    \li SQL_ATTR_ODBC_VERSION
-    \endlist
-
-    \li
-    \list
-    \li CLIENT_COMPRESS
-    \li CLIENT_FOUND_ROWS
-    \li CLIENT_IGNORE_SPACE
-    \li CLIENT_ODBC
-    \li CLIENT_NO_SCHEMA
-    \li CLIENT_INTERACTIVE
-    \li UNIX_SOCKET
-    \li MYSQL_OPT_RECONNECT
-    \li MYSQL_OPT_CONNECT_TIMEOUT
-    \li MYSQL_OPT_READ_TIMEOUT
-    \li MYSQL_OPT_WRITE_TIMEOUT
-    \li MYSQL_OPT_LOCAL_INFILE
-    \li MYSQL_OPT_SSL_KEY
-    \li MYSQL_OPT_SSL_CERT
-    \li MYSQL_OPT_SSL_CA
-    \li MYSQL_OPT_SSL_CAPATH
-    \li MYSQL_OPT_SSL_CIPHER
-    \li MYSQL_OPT_SSL_CRL
-    \li MYSQL_OPT_SSL_CRLPATH
-    \li SSL_KEY (deprecated, use MYSQL_OPT_SSL_KEY)
-    \li SSL_CERT (deprecated, use MYSQL_OPT_SSL_CERT)
-    \li SSL_CA (deprecated, use MYSQL_OPT_SSL_CA)
-    \li SSL_CAPATH (deprecated, use MYSQL_OPT_SSL_CAPATH)
-    \li SSL_CIPHER (deprecated, use MYSQL_OPT_SSL_CIPHER)
-    \endlist
-
-    \li
-    \list
-    \li connect_timeout
-    \li options
-    \li tty
-    \li requiressl
-    \li service
-    \endlist
-
-    \header \li DB2 \li OCI
-    \row
-
-    \li
-    \list
-    \li SQL_ATTR_ACCESS_MODE
-    \li SQL_ATTR_LOGIN_TIMEOUT
-    \endlist
-
-    \li
-    \list
-    \li OCI_ATTR_PREFETCH_ROWS
-    \li OCI_ATTR_PREFETCH_MEMORY
-    \endlist
-
-    \header \li SQLite \li Interbase
-    \row
-
-    \li
-    \list
-    \li QSQLITE_BUSY_TIMEOUT
-    \li QSQLITE_OPEN_READONLY
-    \li QSQLITE_OPEN_URI
-    \li QSQLITE_ENABLE_SHARED_CACHE
-    \li QSQLITE_ENABLE_REGEXP
-    \li QSQLITE_NO_USE_EXTENDED_RESULT_CODES
-    \endlist
-
-    \li
-    \list
-    \li ISC_DPB_LC_CTYPE
-    \li ISC_DPB_SQL_ROLE_NAME
-    \endlist
-
-    \endtable
+    database client used and are described for each plugin in the
+    \l{sql-driver.html}{SQL Database Drivers} page.
 
     Examples:
     \snippet code/src_sql_kernel_qsqldatabase.cpp 4
@@ -1299,6 +1176,11 @@ bool QSqlDatabase::isDriverAvailable(const QString& name)
     \li sqlite *connection
     \li \c qsql_sqlite.cpp
     \row
+    \li QMIMER
+    \li QMimerSQLDriver
+    \li MimerSession *connection
+    \li \c qsql_mimer.cpp
+    \row
     \li QIBASE
     \li QIBaseDriver
     \li isc_db_handle connection
@@ -1342,6 +1224,8 @@ bool QSqlDatabase::isValid() const
 
     \note The new connection has not been opened. Before using the new
     connection, you must call open().
+
+    \reentrant
 */
 QSqlDatabase QSqlDatabase::cloneDatabase(const QSqlDatabase &other, const QString &connectionName)
 {
@@ -1373,19 +1257,7 @@ QSqlDatabase QSqlDatabase::cloneDatabase(const QSqlDatabase &other, const QStrin
 
 QSqlDatabase QSqlDatabase::cloneDatabase(const QString &other, const QString &connectionName)
 {
-    const QConnectionDict *dict = dbDict();
-    Q_ASSERT(dict);
-
-    dict->lock.lockForRead();
-    QSqlDatabase otherDb = dict->value(other);
-    dict->lock.unlock();
-    if (!otherDb.isValid())
-        return QSqlDatabase();
-
-    QSqlDatabase db(otherDb.driverName());
-    db.d->copy(otherDb.d);
-    QSqlDatabasePrivate::addDatabase(db, connectionName);
-    return db;
+    return cloneDatabase(s_sqlGlobals()->connection(other), connectionName);
 }
 
 /*!
