@@ -2,14 +2,24 @@
 // Copyright (C) 2016 Intel Corporation.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
-#include <private/qglobal_p.h> // for the icu feature test
+#include <QDateTime>
 #include <QTest>
-#if QT_CONFIG(timezone)
-#include <QTimeZone>
-#endif
-#include <qdatetime.h>
-#include <qlocale.h>
+
+#include <QLocale>
 #include <QMap>
+#include <QTimeZone>
+
+#include <private/qglobal_p.h> // for the icu feature test
+#include <private/qdatetime_p.h>
+#if !QT_CONFIG(timezone)
+#  include <private/qtenvironmentvariables_p.h> // for qTzName()
+#endif
+
+using namespace QtPrivate::DateTimeConstants;
+
+#if defined(Q_OS_WIN) && !QT_CONFIG(icu)
+#  define USING_WIN_TZ
+#endif
 
 class tst_QDate : public QObject
 {
@@ -34,10 +44,8 @@ private Q_SLOTS:
     void weekNumber_invalid();
     void weekNumber_data();
     void weekNumber();
-#if QT_CONFIG(timezone)
     void startOfDay_endOfDay_data();
     void startOfDay_endOfDay();
-#endif
     void startOfDay_endOfDay_fixed_data();
     void startOfDay_endOfDay_fixed();
     void startOfDay_endOfDay_bounds();
@@ -80,6 +88,15 @@ private Q_SLOTS:
     void qdebug() const;
 private:
     QDate defDate() const { return QDate(1900, 1, 1); }
+
+    QDate epochDate() const {
+        using namespace QtPrivate::DateTimeConstants;
+        Q_ASSERT(JULIAN_DAY_FOR_EPOCH == QDate(1970, 1, 1).toJulianDay());
+        return QDate::fromJulianDay(JULIAN_DAY_FOR_EPOCH);
+    }
+
+    static constexpr qint64 minJd = JulianDayMin;
+    static constexpr qint64 maxJd = JulianDayMax;
     QDate invalidDate() const { return QDate(); }
 };
 
@@ -90,9 +107,6 @@ void tst_QDate::isNull_data()
     QTest::addColumn<qint64>("jd");
     QTest::addColumn<bool>("null");
 
-    qint64 minJd = Q_INT64_C(-784350574879);
-    qint64 maxJd = Q_INT64_C( 784354017364);
-
     QTest::newRow("qint64 min") << std::numeric_limits<qint64>::min() << true;
     QTest::newRow("minJd - 1")  << minJd - 1                          << true;
     QTest::newRow("minJd")      << minJd                              << false;
@@ -480,124 +494,174 @@ void tst_QDate::weekNumber_invalid()
     QCOMPARE( dt.weekNumber( &yearNumber ), 0 );
 }
 
-#if QT_CONFIG(timezone)
+/* The MS backend tends to lack data for historical transitions.  So some of the
+   transition-based tests will get wrong results, that we can't do anything
+   about, when using that backend.  Rather than complicating the #if-ery more,
+   overtly record, in a flags column, which we need to ignore and merely make
+   the testing of these flags subject to #if-ery.
+
+   Android appears to lack at least one other.
+*/
+enum BackendKludge { IgnoreStart = 1, IgnoreEnd = 2, };
+Q_DECLARE_FLAGS(BackendKludges, BackendKludge)
+Q_DECLARE_OPERATORS_FOR_FLAGS(BackendKludges)
+
 void tst_QDate::startOfDay_endOfDay_data()
 {
     QTest::addColumn<QDate>("date"); // Typically a spring-forward.
     // A zone in which that date's start and end are worth checking:
-    QTest::addColumn<QByteArray>("zoneName");
+    QTest::addColumn<QTimeZone>("zone");
     // The start and end times in that zone:
     QTest::addColumn<QTime>("start");
     QTest::addColumn<QTime>("end");
+    // Ignored for backends that don't need it:
+    QTest::addColumn<BackendKludges>("kludge");
 
-    const QTime initial(0, 0), final(23, 59, 59, 999), invalid(QDateTime().time());
-
-    // UTC is always a valid zone.
-    QTest::newRow("epoch")
-        << QDate(1970, 1, 1) << QByteArray("UTC")
-        << initial << final;
-    if (QTimeZone("America/Sao_Paulo").isValid()) {
-        QTest::newRow("Brazil")
-            << QDate(2008, 10, 19) << QByteArray("America/Sao_Paulo")
-            << QTime(1, 0) << final;
-    }
-#if QT_CONFIG(icu) || !defined(Q_OS_WIN) // MS's TZ APIs lack data
-    if (QTimeZone("Europe/Sofia").isValid()) {
-        QTest::newRow("Sofia")
-            << QDate(1994, 3, 27) << QByteArray("Europe/Sofia")
-            << QTime(1, 0) << final;
-    }
+    const QTime early(0, 0), late(23, 59, 59, 999), invalid(QDateTime().time());
+    constexpr BackendKludges Clean = {};
+    constexpr BackendKludges IgnoreBoth = IgnoreStart | IgnoreEnd;
+#ifdef USING_WIN_TZ
+    constexpr BackendKludges MsNoStart = IgnoreStart;
+    constexpr BackendKludges MsNoBoth = IgnoreBoth;
+#else
+    constexpr BackendKludges MsNoStart = Clean;
+    constexpr BackendKludges MsNoBoth = Clean;
+    // And use IgnoreBoth directly for the one transition Android lacks.
 #endif
-    if (QTimeZone("Pacific/Kiritimati").isValid()) {
-        QTest::newRow("Kiritimati")
-            << QDate(1994, 12, 31) << QByteArray("Pacific/Kiritimati")
-            << invalid << invalid;
+    const QTimeZone UTC(QTimeZone::UTC);
+
+    using Bound = std::numeric_limits<qint64>;
+    const auto dateAtMillis = [UTC](qint64 millis) {
+        return QDateTime::fromMSecsSinceEpoch(millis, UTC).date();
+    };
+
+    // UTC and fixed offset are always available and predictable:
+    QTest::newRow("epoch") << epochDate() << UTC << early << late << Clean;
+
+    // First and last days in QDateTime's supported range:
+    QTest::newRow("earliest")
+        << dateAtMillis(Bound::min()) << UTC << invalid << late << Clean;
+    QTest::newRow("latest")
+        << dateAtMillis(Bound::max()) << UTC << early << invalid << Clean;
+
+    const struct {
+        const char *test;
+        const char *zone;
+        const QDate day;
+        const QTime start;
+        const QTime end;
+        const BackendKludges msOpt;
+    } transitions[] = {
+        // The western Mexico time-zones skipped the first hour of 1970.
+        { "BajaMexico", "America/Hermosillo", QDate(1970, 1, 1), QTime(1, 0), late, MsNoStart },
+
+        // Compare tst_QDateTime::fromStringDateFormat(ISO 24:00 in DST).
+        { "Brazil", "America/Sao_Paulo", QDate(2008, 10, 19), QTime(1, 0), late, Clean },
+
+        // Several southern zones within EET (but not the northern ones) spent
+        // part of the 1990s using midnight as spring transition.
+        { "Sofia", "Europe/Sofia", QDate(1994, 3, 27), QTime(1, 0), late, MsNoStart },
+
+        // Two Pacific zones skipped days to get on the west of the
+        // International Date Line; those days have neither start nor end.
+        { "Kiritimati", "Pacific/Kiritimati", QDate(1994, 12, 31), invalid, invalid, IgnoreBoth },
+        { "Samoa", "Pacific/Apia", QDate(2011, 12, 30), invalid, invalid, MsNoBoth },
+
+        // TODO: find other zones with transitions at/crossing midnight.
+    };
+    const QTimeZone local = QTimeZone::LocalTime;
+
+#if QT_CONFIG(timezone)
+    const QTimeZone sys = QTimeZone::systemTimeZone();
+    QVERIFY2(sys.isValid(), "Test depends on properly configured system");
+    for (const auto &tran : transitions) {
+        if (QTimeZone zone(tran.zone); zone.isValid()) {
+            QTest::newRow(tran.test)
+                << tran.day << zone << tran.start << tran.end << tran.msOpt;
+            if (zone == sys) {
+                QTest::addRow("Local=%s", tran.test)
+                    << tran.day << local << tran.start << tran.end << tran.msOpt;
+            }
+        }
     }
-    if (QTimeZone("Pacific/Apia").isValid()) {
-        QTest::newRow("Samoa")
-            << QDate(2011, 12, 30) << QByteArray("Pacific/Apia")
-            << invalid << invalid;
+#else
+    const auto isLocalZone = [](const char *zone) {
+        const QLatin1StringView name(zone);
+        for (int i = 0; i < 2; ++i) {
+            if (qTzName(i) == name)
+                return true;
+        }
+        return false;
+    };
+    for (const auto &tran : transitions) {
+        if (isLocalZone(tran.zone)) { // Might need a different name to match
+            QTest::addRow("Local=%s", tran.test)
+                << tran.day << local << tran.start << tran.end << tran.msOpt;
+        }
     }
-    // TODO: find other zones with transitions at/crossing midnight.
+#endif // timezone
 }
 
 void tst_QDate::startOfDay_endOfDay()
 {
-    QFETCH(QDate, date);
-    QFETCH(QByteArray, zoneName);
-    QFETCH(QTime, start);
-    QFETCH(QTime, end);
-    const QTimeZone zone(zoneName);
+    QFETCH(const QDate, date);
+    QFETCH(const QTimeZone, zone);
+    QFETCH(const QTime, start);
+    QFETCH(const QTime, end);
+#if defined(USING_WIN_TZ) || defined(Q_OS_ANDROID) // Coping with backend limitations.
+    QFETCH(const BackendKludges, kludge);
+#define UNLESSKLUDGE(flag) if (!kludge.testFlag(flag))
+#else
+#define UNLESSKLUDGE(flag)
+#endif
     QVERIFY(zone.isValid());
-    const bool isSystem = QTimeZone::systemTimeZone() == zone;
+
     QDateTime front(date.startOfDay(zone)), back(date.endOfDay(zone));
     if (end.isValid())
         QCOMPARE(date.addDays(1).startOfDay(zone).addMSecs(-1), back);
     if (start.isValid())
         QCOMPARE(date.addDays(-1).endOfDay(zone).addMSecs(1), front);
-    do { // Avoids duplicating these tests for local-time when it *is* zone:
-        if (start.isValid()) {
-            QCOMPARE(front.date(), date);
-            QCOMPARE(front.time(), start);
-        }
-        if (end.isValid()) {
-            QCOMPARE(back.date(), date);
-            QCOMPARE(back.time(), end);
-        }
-        if (front.timeSpec() == Qt::LocalTime)
-            break;
-        front = date.startOfDay(Qt::LocalTime);
-        back = date.endOfDay(Qt::LocalTime);
-    } while (isSystem);
-    if (end.isValid())
-        QCOMPARE(date.addDays(1).startOfDay(Qt::LocalTime).addMSecs(-1), back);
-    if (start.isValid())
-        QCOMPARE(date.addDays(-1).endOfDay(Qt::LocalTime).addMSecs(1), front);
-    if (!isSystem) {
-        // These might fail if system zone coincides with zone; but only if it
-        // did something similarly unusual on the date picked for this test.
-        if (start.isValid()) {
-            QCOMPARE(front.date(), date);
-            QCOMPARE(front.time(), QTime(0, 0));
-        }
-        if (end.isValid()) {
-            QCOMPARE(back.date(), date);
-            QCOMPARE(back.time(), QTime(23, 59, 59, 999));
-        }
+
+    if (start.isValid()) {
+        QVERIFY(front.isValid());
+        QCOMPARE(front.date(), date);
+        UNLESSKLUDGE(IgnoreStart) QCOMPARE(front.time(), start);
+    } else UNLESSKLUDGE(IgnoreStart) {
+        QVERIFY(!front.isValid());
     }
+    if (end.isValid()) {
+        QVERIFY(back.isValid());
+        QCOMPARE(back.date(), date);
+        UNLESSKLUDGE(IgnoreEnd) QCOMPARE(back.time(), end);
+    } else UNLESSKLUDGE(IgnoreEnd) {
+        QVERIFY(!back.isValid());
+    }
+#undef UNLESSKLUDGE
 }
-#endif // timezone
 
 void tst_QDate::startOfDay_endOfDay_fixed_data()
 {
+    QTest::addColumn<QDate>("date");
+
     const qint64 kilo(1000);
     using Bounds = std::numeric_limits<qint64>;
-    const QDateTime
-        first(QDateTime::fromMSecsSinceEpoch(Bounds::min() + 1, Qt::UTC)),
-        start32sign(QDateTime::fromMSecsSinceEpoch(Q_INT64_C(-0x80000000) * kilo, Qt::UTC)),
-        end32sign(QDateTime::fromMSecsSinceEpoch(Q_INT64_C(0x80000000) * kilo, Qt::UTC)),
-        end32unsign(QDateTime::fromMSecsSinceEpoch(Q_INT64_C(0x100000000) * kilo, Qt::UTC)),
-        last(QDateTime::fromMSecsSinceEpoch(Bounds::max(), Qt::UTC));
+    const auto UTC = QTimeZone::UTC;
+    const QDateTime first(QDateTime::fromMSecsSinceEpoch(Bounds::min() + 1, UTC));
+    const QDateTime start32sign(QDateTime::fromMSecsSinceEpoch(Q_INT64_C(-0x80000000) * kilo, UTC));
+    const QDateTime end32sign(QDateTime::fromMSecsSinceEpoch(Q_INT64_C(0x80000000) * kilo, UTC));
+    const QDateTime end32unsign(QDateTime::fromMSecsSinceEpoch(Q_INT64_C(0x100000000) * kilo, UTC));
+    const QDateTime last(QDateTime::fromMSecsSinceEpoch(Bounds::max(), UTC));
 
-    const struct {
-        const char *name;
-        QDate date;
-    } data[] = {
-        { "epoch", QDate(1970, 1, 1) },
-        { "y2k-leap-day", QDate(2000, 2, 29) },
-        { "start-1900", QDate(1900, 1, 1) }, // QTBUG-99747
-        // Just outside the start and end of 32-bit time_t:
-        { "pre-sign32", QDate(start32sign.date().year(), 1, 1) },
-        { "post-sign32", QDate(end32sign.date().year(), 12, 31) },
-        { "post-uint32", QDate(end32unsign.date().year(), 12, 31) },
-        // Just inside the start and end of QDateTime's range:
-        { "first-full", first.date().addDays(1) },
-        { "last-full", last.date().addDays(-1) }
-    };
-
-    QTest::addColumn<QDate>("date");
-    for (const auto &r : data)
-        QTest::newRow(r.name) << r.date;
+    QTest::newRow("epoch") <<  epochDate();
+    QTest::newRow("y2k-leap-day") << QDate(2000, 2, 29);
+    QTest::newRow("start-1900") << QDate(1900, 1, 1); // QTBUG-99747
+   // Just outside the start and end of 32-bit time_t:
+    QTest::newRow("pre-sign32") << QDate(start32sign.date().year(), 1, 1);
+    QTest::newRow("post-sign32") << QDate(end32sign.date().year(), 12, 31);
+    QTest::newRow("post-uint32") << QDate(end32unsign.date().year(), 12, 31);
+    // Just inside the start and end of QDateTime's range:
+    QTest::newRow("first-full") << first.date().addDays(1);
+    QTest::newRow("last-full") << last.date().addDays(-1);
 }
 
 void tst_QDate::startOfDay_endOfDay_fixed()
@@ -605,28 +669,29 @@ void tst_QDate::startOfDay_endOfDay_fixed()
     const QTime early(0, 0), late(23, 59, 59, 999);
     QFETCH(QDate, date);
 
-    QDateTime start(date.startOfDay(Qt::UTC));
-    QDateTime end(date.endOfDay(Qt::UTC));
+    QDateTime start(date.startOfDay(QTimeZone::UTC));
+    QDateTime end(date.endOfDay(QTimeZone::UTC));
     QCOMPARE(start.date(), date);
     QCOMPARE(end.date(), date);
     QCOMPARE(start.time(), early);
     QCOMPARE(end.time(), late);
-    QCOMPARE(date.addDays(1).startOfDay(Qt::UTC).addMSecs(-1), end);
-    QCOMPARE(date.addDays(-1).endOfDay(Qt::UTC).addMSecs(1), start);
+    QCOMPARE(date.addDays(1).startOfDay(QTimeZone::UTC).addMSecs(-1), end);
+    QCOMPARE(date.addDays(-1).endOfDay(QTimeZone::UTC).addMSecs(1), start);
     for (int offset = -60 * 16; offset <= 60 * 16; offset += 65) {
-        start = date.startOfDay(Qt::OffsetFromUTC, offset);
-        end = date.endOfDay(Qt::OffsetFromUTC, offset);
+        const auto zone = QTimeZone::fromSecondsAheadOfUtc(offset);
+        start = date.startOfDay(zone);
+        end = date.endOfDay(zone);
         QCOMPARE(start.date(), date);
         QCOMPARE(end.date(), date);
         QCOMPARE(start.time(), early);
         QCOMPARE(end.time(), late);
-        QCOMPARE(date.addDays(1).startOfDay(Qt::OffsetFromUTC, offset).addMSecs(-1), end);
-        QCOMPARE(date.addDays(-1).endOfDay(Qt::OffsetFromUTC, offset).addMSecs(1), start);
+        QCOMPARE(date.addDays(1).startOfDay(zone).addMSecs(-1), end);
+        QCOMPARE(date.addDays(-1).endOfDay(zone).addMSecs(1), start);
     }
 
     // Minimal testing for LocalTime and TimeZone
-    QCOMPARE(date.startOfDay(Qt::LocalTime).date(), date);
-    QCOMPARE(date.endOfDay(Qt::LocalTime).date(), date);
+    QCOMPARE(date.startOfDay().date(), date);
+    QCOMPARE(date.endOfDay().date(), date);
 #if QT_CONFIG(timezone)
     const QTimeZone cet("Europe/Oslo");
     if (cet.isValid()) {
@@ -640,31 +705,33 @@ void tst_QDate::startOfDay_endOfDay_bounds()
 {
     // Check the days in which QDateTime's range starts and ends:
     using Bounds = std::numeric_limits<qint64>;
+    const auto UTC = QTimeZone::UTC;
     const QDateTime
-        first(QDateTime::fromMSecsSinceEpoch(Bounds::min(), Qt::UTC)),
-        last(QDateTime::fromMSecsSinceEpoch(Bounds::max(), Qt::UTC)),
-        epoch(QDateTime::fromMSecsSinceEpoch(0, Qt::UTC));
+        first(QDateTime::fromMSecsSinceEpoch(Bounds::min(), UTC)),
+        last(QDateTime::fromMSecsSinceEpoch(Bounds::max(), UTC)),
+        epoch(QDateTime::fromMSecsSinceEpoch(0, UTC));
     // First, check these *are* the start and end of QDateTime's range:
     QVERIFY(first.isValid());
     QVERIFY(last.isValid());
     QVERIFY(first < epoch);
     QVERIFY(last > epoch);
-    // QDateTime's addMSecs doesn't check against {und,ov}erflow ...
     QVERIFY(!first.addMSecs(-1).isValid() || first.addMSecs(-1) > first);
     QVERIFY(!last.addMSecs(1).isValid() || last.addMSecs(1) < last);
 
     // Now test start/end methods with them:
-    QCOMPARE(first.date().endOfDay(Qt::UTC).time(), QTime(23, 59, 59, 999));
-    QCOMPARE(last.date().startOfDay(Qt::UTC).time(), QTime(0, 0));
-    QVERIFY(!first.date().startOfDay(Qt::UTC).isValid());
-    QVERIFY(!last.date().endOfDay(Qt::UTC).isValid());
+    QCOMPARE(first.date().endOfDay(UTC).time(), QTime(23, 59, 59, 999));
+    QCOMPARE(last.date().startOfDay(UTC).time(), QTime(0, 0));
+    QVERIFY(!first.date().startOfDay(UTC).isValid());
+    QVERIFY(!last.date().endOfDay(UTC).isValid());
 
     // Test for QTBUG-100873, shouldn't assert:
     const QDate qdteMin(1752, 9, 14); // Used by QDateTimeEdit
-    QCOMPARE(qdteMin.startOfDay(Qt::UTC).date(), qdteMin);
-    QCOMPARE(qdteMin.startOfDay(Qt::LocalTime).date(), qdteMin);
+    QCOMPARE(qdteMin.startOfDay(UTC).date(), qdteMin);
+    QCOMPARE(qdteMin.startOfDay().date(), qdteMin);
 #if QT_CONFIG(timezone)
-    QCOMPARE(qdteMin.startOfDay(QTimeZone::systemTimeZone()).date(), qdteMin);
+    const QTimeZone sys = QTimeZone::systemTimeZone();
+    QVERIFY2(sys.isValid(), "Test depends on properly configured system");
+    QCOMPARE(qdteMin.startOfDay(sys).date(), qdteMin);
     QTimeZone berlin("Europe/Berlin");
     if (berlin.isValid())
         QCOMPARE(qdteMin.startOfDay(berlin).date(), qdteMin);
@@ -675,8 +742,6 @@ void tst_QDate::julianDaysLimits()
 {
     qint64 min = std::numeric_limits<qint64>::min();
     qint64 max = std::numeric_limits<qint64>::max();
-    qint64 minJd = Q_INT64_C(-784350574879);
-    qint64 maxJd = Q_INT64_C( 784354017364);
 
     QDate maxDate = QDate::fromJulianDay(maxJd);
     QDate minDate = QDate::fromJulianDay(minJd);
@@ -899,9 +964,6 @@ void tst_QDate::addYears_data()
 
 void tst_QDate::daysTo()
 {
-    qint64 minJd = Q_INT64_C(-784350574879);
-    qint64 maxJd = Q_INT64_C( 784354017364);
-
     QDate dt1(2000, 1, 1);
     QDate dt2(2000, 1, 5);
     QCOMPARE(dt1.daysTo(dt2), (qint64) 4);
@@ -1196,13 +1258,13 @@ void tst_QDate::fromStringDateFormat_data()
         << QString::fromLatin1(" 13 Feb 1987 13:24:51 +0100")
         << Qt::RFC2822Date << QDate(1987, 2, 13);
     QTest::newRow("RFC 2822 with day") << QString::fromLatin1("Thu, 01 Jan 1970 00:12:34 +0000")
-        << Qt::RFC2822Date << QDate(1970, 1, 1);
+        << Qt::RFC2822Date << epochDate();
     QTest::newRow("RFC 2822 with day after space")
         << QString::fromLatin1(" Thu, 01 Jan 1970 00:12:34 +0000")
-        << Qt::RFC2822Date << QDate(1970, 1, 1);
+        << Qt::RFC2822Date << epochDate();
     // No timezone
     QTest::newRow("RFC 2822 no timezone") << QString::fromLatin1("01 Jan 1970 00:12:34")
-        << Qt::RFC2822Date << QDate(1970, 1, 1);
+        << Qt::RFC2822Date << epochDate();
     // No time specified
     QTest::newRow("RFC 2822 date only") << QString::fromLatin1("01 Nov 2002")
         << Qt::RFC2822Date << QDate(2002, 11, 1);
@@ -1241,7 +1303,7 @@ void tst_QDate::fromStringDateFormat_data()
         << Qt::RFC2822Date << QDate(1987, 2, 13);
     // No timezone
     QTest::newRow("RFC 850 and 1036 no timezone") << QString::fromLatin1("Thu Jan 01 00:12:34 1970")
-        << Qt::RFC2822Date << QDate(1970, 1, 1);
+        << Qt::RFC2822Date << epochDate();
     // No time specified
     QTest::newRow("RFC 850 and 1036 date only") << QString::fromLatin1("Fri Nov 01 2002")
         << Qt::RFC2822Date << QDate(2002, 11, 1);
@@ -1351,11 +1413,23 @@ void tst_QDate::fromStringFormat_data()
     QTest::newRow("lateMarch") << QString("9999-03-06") << QString("yyyy-MM-dd") << QDate(9999, 3, 6);
     QTest::newRow("late") << QString("9999-12-31") << QString("yyyy-MM-dd") << QDate(9999, 12, 31);
 
+    QTest::newRow("quoted-dd") << QString("21dd-05-2006") << QString("dd'dd'-MM-yyyy")
+                               << QDate(2006, 5, 21);
+    QTest::newRow("quoted-MM") << QString("21-MM05-2006") << QString("dd-'MM'MM-yyyy")
+                               << QDate(2006, 5, 21);
+    QTest::newRow("quotes-empty") << QString("21-'05-2006") << QString("dd-MM-''yy")
+                                  << QDate(2006, 5, 21);
+
     // Test unicode handling.
     QTest::newRow("Unicode in format string")
         << QString(u8"2020🤣09🤣21") << QString(u8"yyyy🤣MM🤣dd") << QDate(2020, 9, 21);
-    QTest::newRow("Unicode in quoted format string")
+    QTest::newRow("Unicode-in-format-string-quoted-emoji")
         << QString(u8"🤣🤣2020👍09🤣21") << QString(u8"'🤣🤣'yyyy👍MM🤣dd") << QDate(2020, 9, 21);
+    QTest::newRow("Unicode-in-quoted-dd-format-string")
+            << QString(u8"🤣🤣2020👍09🤣21dd") << QString(u8"🤣🤣yyyy👍MM🤣dd'dd'") << QDate(2020, 9, 21);
+    QTest::newRow("Unicode-in-all-formats-quoted-string")
+            << QString(u8"🤣🤣yyyy2020👍MM09🤣21dd") << QString(u8"🤣🤣'yyyy'yyyy👍'MM'MM🤣dd'dd'")
+            << QDate(2020, 9, 21);
 
     // QTBUG-84334
     QTest::newRow("-ve year: front, nosep")
@@ -1438,6 +1512,8 @@ void tst_QDate::fromStringFormat()
     QFETCH(QDate, expected);
 
     QDate dt = QDate::fromString(string, format);
+    QEXPECT_FAIL("quotes-empty", "QTBUG-110669: doubled single-quotes in format mishandled",
+                 Continue);
     QCOMPARE(dt, expected);
 }
 #endif // datetimeparser
@@ -1650,9 +1726,6 @@ void tst_QDate::roundtrip() const
         loopDate = loopDate.addDays(1);
     }
 
-    qint64 minJd = Q_INT64_C(-784350574879);
-    qint64 maxJd = Q_INT64_C( 784354017364);
-
     // Test Gregorian round trip at top end of conversion range
     loopDate = QDate::fromJulianDay(maxJd);
     while (loopDate.toJulianDay() >= maxJd - 146397) {
