@@ -118,12 +118,14 @@ public:
         if (port > 0)
             cName += QLatin1Char(':') + QString::number(port);
 
+        QString opts = params;
         if (driver == "QSQLITE") {
             // Since the database for sqlite is generated at runtime it's always
             // available, but we use QTempDir so it's always in a different
             // location. Thus, let's ignore the path completely.
             cName = "SQLite";
             qInfo("SQLite will use the database located at %ls", qUtf16Printable(dbName));
+            opts += QStringLiteral(";QSQLITE_ENABLE_NON_ASCII_CASE_FOLDING");
         }
 
         auto db = QSqlDatabase::addDatabase(driver, cName);
@@ -137,7 +139,7 @@ public:
         db.setPassword(passwd);
         db.setHostName(host);
         db.setPort(port);
-        db.setConnectOptions(params);
+        db.setConnectOptions(opts);
         dbNames.append(cName);
     }
 
